﻿import socket
import threading
import tkinter as tk
from tkinter import scrolledtext, messagebox, StringVar
from tkinter import Menu, ttk
import json
import os

class ClientApp:
    def __init__(self, root, host='147.185.221.19', port=42439):
        self.root = root
        self.root.title("DMconnect")
        self.root.geometry("600x400")
        self.host = host
        self.port = port
        self.client_socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.current_chat = "Общий чат"
        self.logged_in = False
        self.logged_in_user = ""
        self.settings_file = 'settings.json'
        self.accounts_file = 'accounts.json'
        self.load_settings()
        self.load_accounts()

        style = ttk.Style()
        style.configure("TButton", padding=6)
        style.configure("TLabel", padding=5)

        self.chat_display = scrolledtext.ScrolledText(root, state='disabled', wrap='word', width=70, height=15, font=("Arial", 12))
        self.chat_display.grid(row=0, column=0, padx=10, pady=10, sticky="nsew")

        self.input_frame = tk.Frame(root)
        self.input_frame.grid(row=1, column=0, padx=10, pady=5, sticky="ew")

        self.input_text = tk.Text(self.input_frame, height=4, wrap='word', font=("Arial", 12))
        self.input_text.pack(side=tk.LEFT, fill=tk.X, expand=True)
        self.input_text.bind("<KeyRelease>", self.update_char_count)
        self.input_text.bind("<Return>", self.handle_return_key)
        self.input_text.bind("<Shift-Return>", self.handle_shift_return_key)

        self.send_button = ttk.Button(self.input_frame, text="Send", command=self.send_message)
        self.send_button.pack(side=tk.RIGHT)

        self.char_count_label = tk.Label(root, text="0/1000", font=("Arial", 10))
        self.char_count_label.grid(row=2, column=0, padx=10, pady=5, sticky="e")

        self.version_label = tk.Label(root, text="DMconnect 0.2", font=("Arial", 10))
        self.version_label.grid(row=2, column=0, padx=10, pady=5, sticky="w")

        self.members_button = ttk.Button(root, text="Пользователи", command=self.request_members)
        self.members_button.grid(row=3, column=0, padx=10, pady=5, sticky="w")

        self.settings_button = ttk.Button(root, text="Настройки", command=self.open_settings_menu)
        self.settings_button.grid(row=3, column=0, padx=10, pady=5, sticky="e")

        self.context_menu = Menu(root, tearoff=0)
        self.context_menu.add_command(label="Копировать", command=self.copy_message)
        self.context_menu.add_command(label="Вставить", command=self.paste_message)
        self.context_menu.add_command(label="Цитировать", command=self.quote_message)
        self.context_menu.add_separator()
        self.context_menu.add_command(label="Настройки", command=self.open_settings_menu)

        self.chat_display.bind("<Button-3>", self.show_context_menu)

        root.grid_rowconfigure(0, weight=1)
        root.grid_rowconfigure(1, weight=0)
        root.grid_columnconfigure(0, weight=1)

        self.max_chars = 1000
        self.stop_event = threading.Event()

        self.start_connection()

    def start_connection(self):
        try:
            self.client_socket.connect((self.host, self.port))
            self.start_receiving_thread()
        except Exception as e:
            self.add_message("Ошибка подключения: " + str(e))

    def add_message(self, message):
        self.chat_display.config(state='normal')
        if ":" in message:
            nickname, msg_content = message.split(":", 1)
            self.chat_display.insert(tk.END, nickname + ":", 'nickname')
            self.chat_display.insert(tk.END, msg_content + "\n")
        else:
            self.chat_display.insert(tk.END, message + '\n')
        self.chat_display.yview(tk.END)
        self.chat_display.config(state='disabled')
        self.chat_display.tag_config('nickname', foreground='red')

    def send_message(self):
        message = self.input_text.get("1.0", tk.END).strip()
        if len(message) <= self.max_chars and message.strip():
            self.send_normal_message(message)
        else:
            self.add_message("Ошибка: сообщение должно содержать не более " + self.max_chars + " символов.")
        self.input_text.delete("1.0", tk.END)
        self.char_count_label.config(text="0/1000")

    def handle_commands(self, message):
        if message.startswith('/login '):
            self.process_login(message)
        elif message.startswith('/register '):
            self.process_registration(message)
        elif message.startswith('/join_server '):
            self.process_join_server(message)
        else:
            self.send_normal_message(message)

    def send_normal_message(self, message):
        if self.logged_in or message.startswith('/login ') or message.startswith('/register '):
            try:
                self.client_socket.send(message.encode('utf-8'))
                if message.lower() == 'exit':
                    self.stop_event.set()
                    self.client_socket.close()
                    self.root.quit()
            except Exception as e:
                self.add_message("Ошибка отправки сообщения: " + e)
        else:
            self.add_message("Ошибка: Не выполнен вход. Используйте команду /login <username> <password> для входа.")

    def process_login(self, message):
        pass

    def process_registration(self, message):
        pass

    def process_join_server(self, message):
        server_name = message[len('/join_server '):].strip()
        if server_name:
            self.client_socket.send("/join_server " + server_name.encode('utf-8'))
        else:
            self.add_message("Ошибка: Укажите имя сервера после команды /join_server.")

    def request_members(self):
        if self.logged_in:
            self.client_socket.send("/members".encode('utf-8'))
        else:
            self.add_message("Ошибка: Не выполнен вход. Пожалуйста, выполните вход.")

    def handle_return_key(self, event):
        if not event.state & 0x0001:
            self.send_message()
        return "break"

    def handle_shift_return_key(self, event):
        self.input_text.insert(tk.END, '\n')
        return "break"

    def update_char_count(self, event=None):
        current_length = len(self.input_text.get("1.0", tk.END)) - 1
        self.char_count_label.config(text=str(current_length) + "/" + str(self.max_chars))

    def receive_messages(self):
        while not self.stop_event.is_set():
            try:
                message = self.client_socket.recv(1024).decode('utf-8').strip()
                if not message:
                    break
                self.process_received_message(message)
            except Exception as e:
                if not self.stop_event.is_set():
                    self.add_message("Ошибка при получении сообщения: " + e)
                break

    def process_received_message(self, message):
        if message.startswith("Login successful."):
            self.logged_in = True
            self.add_message("Вход выполнен успешно!")
        elif message.startswith("Invalid username or password."):
            self.add_message("Ошибка входа: неверные учетные данные.")
        elif message.startswith("Registration successful. Please log in."):
            self.add_message("Регистрация успешна! Пожалуйста, войдите.")
        elif message.startswith("Username already taken. Try another."):
            self.add_message("Имя пользователя уже занято. Попробуйте другое.")
        elif message.startswith("Available servers:"):
            self.add_message(message)
        elif message.startswith("Joined server '"):
            self.add_message(message)
        elif message.startswith("Server does not exist."):
            self.add_message(message)
        else:
            self.add_message(message)

    def start_receiving_thread(self):
        receive_thread = threading.Thread(target=self.receive_messages, daemon=True)
        receive_thread.start()

    def show_context_menu(self, event):
        try:
            self.context_menu.post(event.x_root, event.y_root)
        except tk.TclError:
            pass

    def copy_message(self):
        try:
            selected_text = self.chat_display.get(tk.SEL_FIRST, tk.SEL_LAST)
            self.root.clipboard_clear()
            self.root.clipboard_append(selected_text)
        except tk.TclError:
            pass

    def paste_message(self):
        try:
            clipboard_text = self.root.clipboard_get()
            self.input_text.insert(tk.END, clipboard_text)
        except tk.TclError:
            pass

    def quote_message(self):
        try:
            selected_text = self.chat_display.get(tk.SEL_FIRST, tk.SEL_LAST)
            self.input_text.insert(tk.END, "'" + selected_text + "\n")
        except tk.TclError:
            pass

    def open_settings_menu(self):
        self.settings_window = tk.Toplevel(self.root)
        self.settings_window.title("Настройки")

        self.notifications_var = tk.BooleanVar(value=self.notifications_enabled)
        self.notifications_checkbox = ttk.Checkbutton(self.settings_window, text="Включить уведомления", variable=self.notifications_var, command=self.toggle_notifications)
        self.notifications_checkbox.pack(padx=10, pady=10)

        self.save_button = ttk.Button(self.settings_window, text="Сохранить", command=self.save_settings)
        self.save_button.pack(pady=10)

    def save_settings(self):
        self.notifications_enabled = self.notifications_var.get()
        settings = {'notifications_enabled': self.notifications_enabled}
        with open(self.settings_file, 'w') as f:
            json.dump(settings, f)
        self.add_message("Настройки сохранены.")

    def load_settings(self):
        if os.path.exists(self.settings_file):
            with open(self.settings_file, 'r') as f:
                settings = json.load(f)
                self.notifications_enabled = settings.get('notifications_enabled', True)

    def load_accounts(self):
        if os.path.exists(self.accounts_file):
            with open(self.accounts_file, 'r') as f:
                self.accounts = json.load(f)
        else:
            self.accounts = []

    def save_accounts(self):
        with open(self.accounts_file, 'w') as f:
            json.dump(self.accounts, f)

    def open_login_window(self):
        self.login_window = tk.Toplevel(self.root)
        self.login_window.title("Вход")
        self.login_window.attributes("-topmost", True)

        self.login_var = StringVar()
        self.password_var = StringVar()

        ttk.Label(self.login_window, text="Имя пользователя:").pack(padx=10, pady=5)
        self.login_entry = ttk.Entry(self.login_window, textvariable=self.login_var)
        self.login_entry.pack(padx=10, pady=5)

        ttk.Label(self.login_window, text="Пароль:").pack(padx=10, pady=5)
        self.password_entry = ttk.Entry(self.login_window, show='*', textvariable=self.password_var)
        self.password_entry.pack(padx=10, pady=5)

        self.account_select = ttk.Combobox(self.login_window, values=self.accounts)
        self.account_select.pack(padx=10, pady=5)
        self.account_select.bind("<<ComboboxSelected>>", self.load_selected_account)

        save_account_button = ttk.Button(self.login_window, text="Сохранить аккаунт", command=self.save_account)
        save_account_button.pack(padx=10, pady=5)

        login_button = ttk.Button(self.login_window, text="Войти", command=self.perform_login)
        login_button.pack(padx=10, pady=5)

        register_button = ttk.Button(self.login_window, text="Зарегистрироваться", command=self.perform_registration)
        register_button.pack(padx=10, pady=5)

        self.login_window.protocol("WM_DELETE_WINDOW", self.login_window.withdraw)

    def perform_login(self):
        username = self.login_var.get()
        password = self.password_var.get()
        if username and password:
            self.client_socket.send("/login " + username + " " + password + "".encode('utf-8'))
            self.login_window.destroy()  # Close the login window after trying to log in
        else:
            messagebox.showwarning("Input Error", "Пожалуйста, введите имя пользователя и пароль.")

    def perform_registration(self):
        username = self.login_var.get()
        password = self.password_var.get()
        if username and password:
            self.client_socket.send("/register " + username + " " +  password + "".encode('utf-8'))
            self.login_window.destroy()  # Close the registration window after trying
        else:
            messagebox.showwarning("Input Error", "Пожалуйста, введите имя пользователя и пароль.")

    def load_selected_account(self, event):
        selected_account = self.account_select.get()
        if selected_account:
            username, password = selected_account.split(':')
            self.login_var.set(username)
            self.password_var.set(password)

    def save_account(self):
        username = self.login_var.get()
        password = self.password_var.get()
        if username and password:
            account_info = username + ":" + password
            if account_info not in self.accounts:
                self.accounts.append(account_info)
                self.save_accounts()
                self.account_select['values'] = self.accounts  # Update the dropdown list
                messagebox.showinfo("Success", "Учетная запись сохранена.")
            else:
                messagebox.showwarning("Warning", "Эта учетная запись уже сохранена.")
        else:
            messagebox.showwarning("Input Error", "Пожалуйста, введите имя пользователя и пароль.")

if __name__ == "__main__":
    root = tk.Tk()
    app = ClientApp(root)
    root.mainloop()
